## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Original TechConquest Concept by Bhruic, Modified by Sevo
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

# globals
gc = CyGlobalContext()

PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

class TechConquest:
	def onCityAcquired(self, argsList):
		owner,playerType,city,bConquest,bTrade = argsList
		if bConquest:
			dice = gc.getGame().getMapRand()
			loserTeam = gc.getTeam(owner)
			conqTeam = gc.getTeam(city.getOwner())
			listTechDiff = []
			tradeData = TradeData()
			tradeData.ItemType = TradeableItems.TRADE_TECHNOLOGIES
			for iLoopTech in range(gc.getNumTechInfos()):
				tradeData.iData = iLoopTech
				if (loserTeam.isHasTech(iLoopTech) == True) and (conqTeam.isHasTech(iLoopTech) == False):
					listTechDiff.append(iLoopTech)
			if len(listTechDiff) > 0:
				randTech = dice.get(len(listTechDiff), "Tech Chooser - TechConquest PYTHON")
				techData = gc.getTechInfo(listTechDiff[randTech])
				techCost = techData.getResearchCost()
				baseMultiplier = city.getPopulation()
				
				# Tech gained will be 5% to 100%
				
				# Base Points: will be  5% per population in city (City size 20 will always grant full tech)...
				baseTechPointsAdded = (techCost / 20) * baseMultiplier
				
				# Extra Tech points: will be random up to 50% the total tech cost
				extraTechPointsAdded = dice.get((techCost/2), "Tech Chooser - TechConquest PYTHON")
				
				# The least you could get would be 10% (from city size 2)
				# City size 5 would give 25-75%
				# City size 10 gives 50-100%
				
				totalTechPointsAdded = baseTechPointsAdded + extraTechPointsAdded
				currentTechPoints = conqTeam.getResearchProgress(listTechDiff[randTech])
				
				# Actually grant the technology here:
				if (totalTechPointsAdded > (techCost - currentTechPoints)):
					totalTechPoints = techCost
				else:
					totalTechPoints = currentTechPoints + totalTechPointsAdded
				conqTeam.changeResearchProgress(listTechDiff[randTech], totalTechPoints, city.getOwner())
				
				if gc.getPlayer(city.getOwner()).isHuman():
					CyInterface().addMessage(city.getOwner(),True,20,"From the citizens of %s you learn some of the secrets of %s! (%s gained) points)" %(city.getName(),techData.getDescription(), ((totalTechPointsAdded/techCost)*100) ),'',1,techData.getButton(),ColorTypes(10),city.getX(),city.getY(),True,True)	

class ReInitInterceptorMision:

	def __init__(self):
		self.firsttime = true

	def doReInit(self):
		iPlayer = PyPlayer(CyGame().getActivePlayer())
		unitList = iPlayer.getUnitList()
		# Loop for all players cities
		for ii in range(len(unitList)):
			pLoopUnit = unitList[ii]
			# get the group the unit belongs to. 
			pGroup = pLoopUnit.getGroup()
			# check if unit is doing air patrol
			if (pGroup.getActivityType() == ActivityTypes.ACTIVITY_INTERCEPT):
				# remove mission (this one shold be obsolete, but who knows ;)
				pGroup.popMission()
				pPlot = pGroup.plot()
				# add new mission -> fortify
				pGroup.pushMission(MissionTypes.MISSION_FORTIFY, 0, 0, 0, false, false, MissionAITypes.NO_MISSIONAI, pPlot, pLoopUnit)
				# add new mission -> air patrol
				pGroup.pushMission(MissionTypes.MISSION_AIRPATROL, 0, 0, 0, false, false, MissionAITypes.NO_MISSIONAI, pPlot, pLoopUnit)


class CulturalDecay:
	def getMaxCulture3x3(self, iXOrig, iYOrig, pMap, iPlayer):
		iRetCulture = 0
		for iX in range(3):
			for iY in range(3):
				iXPt = iXOrig + iX - 1
				iYPt = iYOrig + iY - 1
				if (iXPt < 0) or (iXPt >= pMap.getGridWidth()):
					continue
				if (iYPt < 0) or (iYPt >= pMap.getGridHeight()):
					continue
				if (iXPt == iXOrig) and (iYPt == iYOrig):
					continue
				iLclCulture = pMap.plot(iXPt, iYPt).getCulture(iPlayer)
				if (iLclCulture > iRetCulture):
					iRetCulture = iLclCulture
		return iRetCulture

	def onEndGameTurn(self, argsList):
		pMap = CyMap()
		for iX in range(pMap.getGridWidth()):
			for iY in range(pMap.getGridHeight()):
				pPlot = pMap.plot(iX, iY)
				if (pPlot.countTotalCulture() > 0):
					for iLoop in range(gc.getMAX_PLAYERS()):
						pPlayer = gc.getPlayer(iLoop)
						if (pPlayer.isAlive()) or (pPlayer.isBarbarian()):
							iCulture = pPlot.getCulture(iLoop)
							iNumCultureCities = pPlot.getNumCultureRangeCities(iLoop)
							if (iCulture > 10) and (iNumCultureCities == 0):
								iCultureDecay = (int)(iCulture * 0.05)
								if (iCultureDecay < 1):
									iCultureDecay = 1
								pPlot.changeCulture(iLoop, -iCultureDecay, True)
								#pPlot.changeCulture(iLoop, -iCultureDecay)

									
class RealFort:
	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit = argsList

		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")
		
		if (pPlot.getImprovementType() == impFort):
			uct = pUnit.getUnitCombatType()
			if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
				pUnit.setHasPromotion(promFort1, true)
				pTeam = gc.getTeam(pUnit.getTeam())
				pFort = gc.getPromotionInfo(promFort2)
				if (pTeam.isHasTech(pFort.getTechPrereq())):
					pUnit.setHasPromotion(promFort2, true)
				pFort = gc.getPromotionInfo(promFort3)
				if (pTeam.isHasTech(pFort.getTechPrereq())):
					pUnit.setHasPromotion(promFort3, true)

		else:
			if (pUnit.isHasPromotion(promFort1)):
				pUnit.setHasPromotion(promFort1, false)
			if (pUnit.isHasPromotion(promFort2)):
				pUnit.setHasPromotion(promFort2, false)
			if (pUnit.isHasPromotion(promFort3)):
				pUnit.setHasPromotion(promFort3, false)
	
	
	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList

		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")

		pMap = CyMap()
		pPlot = pMap.plot(iX, iY)
		if (iImprovement == impFort):
			for iUnit in range(pPlot.getNumUnits()):
				pUnit = pPlot.getUnit(iUnit)
				uct = pUnit.getUnitCombatType()
				if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
					pUnit.setHasPromotion(promFort1, true)
					pTeam = gc.getTeam(pUnit.getTeam())
					pFort = gc.getPromotionInfo(promFort2)
					if (pTeam.isHasTech(pFort.getTechPrereq())):
						pUnit.setHasPromotion(promFort2, true)
					pFort = gc.getPromotionInfo(promFort3)
					if (pTeam.isHasTech(pFort.getTechPrereq())):
						pUnit.setHasPromotion(promFort3, true)
		else:
			for iUnit in range(pPlot.getNumUnits()):
				pUnit = pPlot.getUnit(iUnit)
				if (pUnit.isHasPromotion(promFort1)):
					pUnit.setHasPromotion(promFort1, false)
				if (pUnit.isHasPromotion(promFort2)):
					pUnit.setHasPromotion(promFort2, false)
				if (pUnit.isHasPromotion(promFort3)):
					pUnit.setHasPromotion(promFort3, false)

	def onTechAcquired(self, argsList):
		iTechType, iTeam, iPlayer, bAnnounce = argsList
	
		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")

		pFort2 = gc.getPromotionInfo(promFort2)
		pFort3 = gc.getPromotionInfo(promFort3)
		if (iTechType == pFort2.getTechPrereq()) or (iTechType == pFort3.getTechPrereq()):
			pMap = CyMap()
			for iX in range(pMap.getGridWidth()):
				for iY in range(pMap.getGridHeight()):
					pPlot = pMap.plot(iX, iY)
					if (pPlot.getImprovementType() == impFort):
						for iUnit in range(pPlot.getNumUnits()):
							pUnit = pPlot.getUnit(iUnit)
							if (pUnit.getTeam() == iTeam):
								uct = pUnit.getUnitCombatType()
								if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
									pUnit.setHasPromotion(promFort1, true)
									pTeam = gc.getTeam(pUnit.getTeam())
									if (iTechType == pFort2.getTechPrereq()):
										pUnit.setHasPromotion(promFort2, true)
									if (iTechType == pFort3.getTechPrereq()):
										pUnit.setHasPromotion(promFort3, true)
										
class HeroicUnits:						
	def onCombatResult(self, argsList):
		heroUnit = argsList[0]
		playerX = PyPlayer(heroUnit.getOwner())
		unitX = PyInfo.UnitInfo(heroUnit.getUnitType())
		# If you set it to 100 a unit has a 1 in 100 chance of getting a promotion, if it meets the minimum exp - see below... 
		# Setting it to 1 means units gain a promotion every battle (1 in 1 chance)
		heroChance = 10
		# Only units with exp higher than this number will be able to get the respective promotion.
		# For example if hero1 is 5, a unit must have 6 or more exp to get the promotion.
		hero1 = 5
		hero2 = 10
		hero3 = 20
		hero4 = 35
		hero5 = 55
		if (heroChance < 1):
			heroChance = 1
		promHero1 = gc.getInfoTypeForString("PROMOTION_HERO1")
		promHero2 = gc.getInfoTypeForString("PROMOTION_HERO2")
		promHero3 = gc.getInfoTypeForString("PROMOTION_HERO3")
		promHero4 = gc.getInfoTypeForString("PROMOTION_HERO4")
		promHero5 = gc.getInfoTypeForString("PROMOTION_HERO5")
		if (heroUnit.getExperience() > hero1):
			iRandNum = gc.getGame().getMapRand().get(heroChance, "Heroic Units")
			if (iRandNum == 0):
				if ((heroUnit.isHasPromotion(promHero1)==false) and (heroUnit.isHasPromotion(promHero2)==false) and (heroUnit.isHasPromotion(promHero3)==false) and (heroUnit.isHasPromotion(promHero4)==false) and (heroUnit.isHasPromotion(promHero5)==false)):
					heroUnit.setHasPromotion(promHero1, true)
					CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'An Exceptional %s (level 1) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()),'',1,'Art/Interface/Buttons/Promotions/Hero1.dds',ColorTypes(8),heroUnit.getX(),heroUnit.getY(),True,True)
					CyInterface().addCombatMessage(CyGame().getActivePlayer(),'An Exceptional %s (level 1) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()))
				elif ((heroUnit.isHasPromotion(promHero1)==true) and (heroUnit.getExperience() > hero2)):
					heroUnit.setHasPromotion(promHero1, false)
					heroUnit.setHasPromotion(promHero2, true)
					CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'An Extraordinary %s (level 2) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()),'',1,'Art/Interface/Buttons/Promotions/Hero2.dds',ColorTypes(8),heroUnit.getX(),heroUnit.getY(),True,True)
					CyInterface().addCombatMessage(CyGame().getActivePlayer(),'An Extraordinary %s (level 2) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()))
				elif ((heroUnit.isHasPromotion(promHero2)==true) and (heroUnit.getExperience() > hero3)):
					heroUnit.setHasPromotion(promHero2, false)
					heroUnit.setHasPromotion(promHero3, true)
					CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'A Hero %s (level 3) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()),'AS2D_UNIT_GREATPEOPLE',1,'Art/Interface/Buttons/Promotions/Hero3.dds',ColorTypes(8),heroUnit.getX(),heroUnit.getY(),True,True)
					CyInterface().addCombatMessage(CyGame().getActivePlayer(),'An Hero %s (level 3) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()))
				elif ((heroUnit.isHasPromotion(promHero3)==true) and (heroUnit.getExperience() > hero4)):
					heroUnit.setHasPromotion(promHero3, false)
					heroUnit.setHasPromotion(promHero4, true)
					CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'A Legendary Hero %s (level 4) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()),'AS2D_UNIT_GREATPEOPLE',1,'Art/Interface/Buttons/Promotions/Hero4.dds',ColorTypes(8),heroUnit.getX(),heroUnit.getY(),True,True)
					CyInterface().addCombatMessage(CyGame().getActivePlayer(),'An Legendary Hero %s (level 4) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()))
				elif ((heroUnit.isHasPromotion(promHero4)==true) and (heroUnit.getExperience() > hero5)):
					heroUnit.setHasPromotion(promHero4, false)
					heroUnit.setHasPromotion(promHero5, true)
					CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'An Avatar %s (level 5) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()),'AS2D_UNIT_GREATPEOPLE',1,'Art/Interface/Buttons/Promotions/Hero5.dds',ColorTypes(8),heroUnit.getX(),heroUnit.getY(),True,True)
					CyInterface().addCombatMessage(CyGame().getActivePlayer(),'An Avatar %s (level 5) of %s was born!' %(unitX.getDescription(), playerX.getCivilizationName()))

class StackAid:
	def updatePlot(self, iX, iY):
		promScoutAid = gc.getInfoTypeForString("PROMOTION_SCOUT_AID")
		promMeleeAid = gc.getInfoTypeForString("PROMOTION_MELEE_AID")
		promArcherAid = gc.getInfoTypeForString("PROMOTION_ARCHER_AID")
		promMountedAid = gc.getInfoTypeForString("PROMOTION_MOUNTED_AID")
		promSiegeAid = gc.getInfoTypeForString("PROMOTION_SIEGE_AID")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")
		uctHelicopter = gc.getInfoTypeForString("UNITCOMBAT_HELICOPTER")

		aScout = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		aMelee = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		aArcher = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		aMounted = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		aSiege = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]
		pPlot = gc.getMap().plot(iX, iY)

		for i in range(0, pPlot.getNumUnits()):
			pUnit = pPlot.getUnit(i)
			uct = pUnit.getUnitCombatType()
			if (pUnit.isDead() == false):
				if (uct == uctRecon) or (uct == uctHelicopter):
					aScout[pUnit.getOwner()] += pUnit.baseCombatStr()
				elif (uct == uctMelee) or (uct == uctArmor):
					aMelee[pUnit.getOwner()] += pUnit.baseCombatStr()
				elif (uct == uctArcher) or (uct == uctGun):
					aArcher[pUnit.getOwner()] += pUnit.baseCombatStr()
				elif (uct == uctMounted):
					aMounted[pUnit.getOwner()] += pUnit.baseCombatStr()
				elif (uct == uctSiege):
					aSiege[pUnit.getOwner()] += pUnit.baseCombatStr()

		for i in range(0, pPlot.getNumUnits()):
			pUnit = pPlot.getUnit(i)
			uct = pUnit.getUnitCombatType()

			pUnit.setHasPromotion(promScoutAid, false)
			pUnit.setHasPromotion(promMeleeAid, false)
			pUnit.setHasPromotion(promArcherAid, false)
			pUnit.setHasPromotion(promMountedAid, false)
			pUnit.setHasPromotion(promSiegeAid, false)
			
			if (uct == uctRecon) or (uct == uctHelicopter):
				totStr = aScout[pUnit.getOwner()] - pUnit.baseCombatStr()
			else:
				totStr = aScout[pUnit.getOwner()]
			if (totStr*2 > pUnit.baseCombatStr()):
				pUnit.setHasPromotion(promScoutAid, true)
			
			if (uct == uctMelee) or (uct == uctArmor):
				totStr = aMelee[pUnit.getOwner()] - pUnit.baseCombatStr()
			else:
				totStr = aMelee[pUnit.getOwner()]
			if (totStr*2 > pUnit.baseCombatStr()):
				pUnit.setHasPromotion(promMeleeAid, true)

			if (uct == uctArcher) or (uct == uctGun):
				totStr = aArcher[pUnit.getOwner()] - pUnit.baseCombatStr()
			else:
				totStr = aArcher[pUnit.getOwner()]
			if (totStr*2 > pUnit.baseCombatStr()):
				pUnit.setHasPromotion(promArcherAid, true)
				
			if (uct == uctMounted):
				totStr = aMounted[pUnit.getOwner()] - pUnit.baseCombatStr()
			else:
				totStr = aMounted[pUnit.getOwner()]
			if (totStr*2 > pUnit.baseCombatStr()):
				pUnit.setHasPromotion(promMountedAid, true)
				
			if (uct == uctSiege):
				totStr = aSiege[pUnit.getOwner()] - pUnit.baseCombatStr()
			else:
				totStr = aSiege[pUnit.getOwner()]
			if (totStr*2 > pUnit.baseCombatStr()):
				pUnit.setHasPromotion(promSiegeAid, true)

	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit = argsList	
		aidX = pPlot.getX()
		aidY = pPlot.getY()
		
		self.updatePlot(aidX, aidY)
		self.updatePlot(aidX+1, aidY)
		self.updatePlot(aidX+1, aidY+1)
		self.updatePlot(aidX, aidY+1)
		self.updatePlot(aidX-1, aidY+1)
		self.updatePlot(aidX-1, aidY)
		self.updatePlot(aidX-1, aidY-1)
		self.updatePlot(aidX, aidY-1)
		self.updatePlot(aidX+1, aidY-1)
		
	def onUnitBuilt(self, argsList):
		pCity,pUnit = argsList	
		aidX = pUnit.getX()
		aidY = pUnit.getY()		
		self.updatePlot(aidX, aidY)
	
	def onUnitCreated(self, argsList):
		pUnit = argsList	
		aidX = pUnit.getX()
		aidY = pUnit.getY()		
		popup2 = PyPopup.PyPopup()
		popup2.setBodyString('Created at: x' + str(aidX) + ' y' + str(aidY))
		popup2.launch()
		#self.updatePlot(aidX, aidY)

	def onUnitKilled(self, argsList):	
		pUnit,iAttacker = argsList	
		aidX = pUnit.getX()
		aidY = pUnit.getY()		
		
		self.updatePlot(aidX, aidY)
